/**
 * Downloads JS files and runs it
 * @param {String} version - Version string of your game
 * @param {Function} onReady - Called when patcher has ended
 */
(function () {
    var Xhr = function (settings) {
        /*settings = {
            get: String, // url to get
            post: String, // url to post
            params: {}, // additional parameters
            onSuccess: Function (res),
            onFail: Function (res),
        }*/
        var xhr = new XMLHttpRequest(),
            getPostBody = function () {
                // turns the params string into a url encoded form
                var param,
                    params = settings.params,
                    first = true,
                    url = '';
                for (param in params) {
                    if (!params.hasOwnProperty(param)) {
                        continue;
                    }

                    if (!first) {
                        url += '&'
                    }
                    url += param + '=' + params[param];
                    first = false;
                }
                return url;
            },
            addHeaders = function () {
                var key,
                    headers = settings.headers;
                for (key in headers) {
                    if (!headers.hasOwnProperty(key)) {
                        continue;
                    }
                    xhr.setRequestHeader(key, headers[key]);
                }
            };

        if (settings.get) {
            xhr.open('GET', settings.get, true);
        } else if (settings.post) {
            xhr.open('POST', settings.post, true);
            if (settings.json) {
                xhr.setRequestHeader('Content-Type', 'text/plain');
                xhr.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
            } else {
                xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            }
        }

        if (settings.headers) {
            addHeaders();
        }

        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                if (xhr.status >= 200 && xhr.status < 300) {
                    if (!settings.onSuccess) {
                        return;
                    }
                    try {
                        settings.onSuccess(JSON.parse(xhr.responseText));
                    } catch (e) {
                        settings.onSuccess({
                            response: xhr.responseText
                        });
                    }
                } else {
                    if (!settings.onFail) {
                        return;
                    }
                    try {
                        settings.onFail(JSON.parse(xhr.responseText));
                    } catch (e) {
                        settings.onFail({
                            response: xhr.responseText
                        });
                    }
                }
            }
        };

        //console.log('success', xhr.responseText);
        if (settings.json) {
            xhr.send(JSON.stringify(settings.params));
        } else if (settings.post) {
            xhr.send(getPostBody());
        } else {
            xhr.send();
        }

        return xhr;
    };
    window.patcher = function (baseUrl, version, onReady) {
        var timeout = 5000, // should complete in 5 seconds
            hasTimedout = false,
            hasFinished = false,
            currentPatch,
            // step 1
            downloadSettings = function () {
                if (hasTimedout) {
                    // too late!
                    return;
                }
                new Xhr({
                    get: baseUrl + '/settings.json',
                    onSuccess: function (res) {
                        var patchVersion = res[version] || 0;

                        if (patchVersion === -1) {
                            // wipe patch!
                            localStorage.setItem('patch_' + version, null);
                            onReady('Wiped clean.');
                        } else if (patchVersion > currentPatch.version) {
                            // download new patch
                            downloadPatch(patchVersion);
                        } else {
                            // equal patch, apply current
                            applyPatch();
                        }
                    },
                    onFail: applyPatch
                });
            },
            // step 2
            downloadPatch = function (patchVersion) {
                if (hasTimedout) {
                    // too late!
                    return;
                }
                var fileName = version + '-' + patchVersion + '.json';
                new Xhr({
                    get: baseUrl + '/' + fileName,
                    onSuccess: function (res) {
                        // store patch
                        currentPatch.version = patchVersion;
                        currentPatch.data = res.data;

                        localStorage.setItem('patch_' + version, JSON.stringify(currentPatch));
                        applyPatch();
                    },
                    onFail: applyPatch
                });
            },
            // step 3
            applyPatch = function () {
                var code;

                if (hasFinished) {
                    // already applied the patch (possibly due to time out)
                    return;
                }
                hasFinished = true;

                // there is no patch
                if (!currentPatch.data) {
                    onReady('Nothing to do.');
                    return;
                }

                // !!
                try {
                    // decompress code
                    code = atob(currentPatch.data);
                    eval(code);
                    onReady('Success');
                } catch (e) {
                    onReady('Failed with', e);
                }
            };

        setTimeout(function () {
            if (hasFinished) {
                return;
            }
            hasTimedout = true;
            console.log('WARNING: Download timed out.');
            applyPatch();
        }, timeout);

        // load currentpatch
        try {
            // try to parse the current patch
            currentPatch = JSON.parse(localStorage.getItem('patch_' + version)) || {
                version: 0
            };
        } catch (e) {
            // something went horribly wrong...
            currentPatch = {
                version: 0
            };
        }

        // start by trying to download settings
        downloadSettings();
    }

})();